# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
#              Nicholas Skaggs <nicholas.skaggs@canonical.com>

import logging
from time import sleep

from autopilot import logging as autopilot_logging


from ubuntuuitoolkit import emulators as toolkit_emulators


logger = logging.getLogger(__name__)


class ClockEmulatorException(toolkit_emulators.ToolkitEmulatorException):
    """Exception raised when there is an error with the emulator."""


class MainView(toolkit_emulators.MainView):

    @autopilot_logging.log_action(logger.info)
    def open_timer(self):
        """Open the Timer tab.

        :return: the Timer page.

        """
        self.switch_to_tab('TimerTab')
        return self.wait_select_single(TimerPage)

    @autopilot_logging.log_action(logger.info)
    def open_alarm(self):
        """Open the Alarm Tab.

        :return: the Alarm page.

        """
        self.switch_to_tab('AlarmTab')
        return self.wait_select_single(AlarmPage)

    # Common Emulator Functions
    def _drag_page(self, page, direction):
        """Function to drag the page up/down"""
        page = self.wait_select_single(page)
        self._wait_to_stop_moving(page)

        x, y, w, h = page.globalRect
        tx = x + (w / 2)
        ty = y + (h / 2)

        if direction == "down":
            self.pointing_device.drag(tx, ty, tx, ty + h / 3)
        else:
            self.pointing_device.drag(tx, ty, tx, ty - h / 3)

        self._wait_to_stop_moving(page)

    def drag_page_up(self, page):
        """Drag the given page up."""
        self._drag_page(page, "up")

    def drag_page_down(self, page):
        """Drag the given page down."""
        self._drag_page(page, "down")

    def longpress_object(self, obj):
        """Function to longpress an object"""
        self.pointing_device.move_to_object(obj)
        self.pointing_device.press()
        sleep(2)
        self.pointing_device.release()

    def _wait_to_stop_moving(self, page):
        page.select_single(
            'QQuickFlickable',
            objectName='animationContainer').moving.wait_for(False)

    # CLOCK Page Emulator Functions
    def get_clock_page(self):
        """Return the page containing the main clock."""
        return self.select_single("ClockPage")

    def get_time_label(self):
        """Return the label with the timer countdown"""
        return self.select_single("ClockPage").wait_select_single(
            "Label", objectName="label")

    # WORLD CLOCK Page Emulator Functions
    def get_world_cities_page(self):
        """Return the page containing the world cities."""
        return self.wait_select_single("WorldClock")

    def get_search_box(self):
        """Returns the search box used to search for world cities online."""
        return self.wait_select_single(
            'SearchBox', objectName="worldSearchBox")

    def get_world_cities_list(self):
        """Return the component of the list of available world cities."""
        return self.wait_select_single(objectName="worldList")

    def get_city_name(self, city):
        """Return the name of the city passed as argument"""
        return city.wait_select_single('Label', objectName='city_name').text

    def get_saved_cities_list(self):
        """Return the component of the user saved world city locations."""
        return self.wait_select_single(objectName="listWorldClocks")

    def grab_first_listed_city(self):
        """Return the first city from the available world cities.

        It returns both local and online.

        """
        return self.wait_select_single("Base", objectName="worldcity0")

    def select_city(self, city):
        """Click the selected city passed as argument"""
        self.pointing_device.click_object(city)

    def grab_first_saved_city(self):
        """Return the first city from the user saved world cities list"""
        return self.wait_select_single(
            "Standard", objectName="savedworldcity0")

    # STOPWATCH Page Emulator Functions
    def get_stopwatch_label(self):
        """Returns the select for the stopwatch label"""
        return self.select_single(
            "StopwatchPage").wait_select_single("Label", objectName="label")

    def get_stopwatch_button(self):
        """Returns the select for the stopwatch button"""
        return self.wait_select_single(
            "StopwatchFace", objectName="stopwatchFace")

    def get_lap_button(self):
        """Returns the select for the lap button"""
        return self.wait_select_single("ImageButton", objectName="lapButton")

    def get_reset_button(self):
        """Returns the select for the reset button"""
        return self.wait_select_single("ImageButton", objectName="resetButton")

    def get_num_of_laps(self):
        """Returns the number of laps in the stopwatch page."""
        stopwatch_page = self.select_single("StopwatchPage")
        return int(stopwatch_page.wait_select_single("QQuickListView").count)

    def get_first_laps_list_item(self):
        """Returns the first lap list item in the stopwatch page."""
        return self.wait_select_single("Standard", objectName="laps0")


class LabelDots(toolkit_emulators.TextField):
    """Autopilot helper for the LabelDots component."""


class Page(toolkit_emulators.UbuntuUIToolkitEmulatorBase):
    """Autopilot helper for Pages."""

    def __init__(self, *args):
        super(Page, self).__init__(*args)
        # XXX we need a better way to keep reference to the main view.
        # --elopio - 2014-01-31
        self.main_view = self.get_root_instance().select_single(MainView)

    def drag_page_up(self):
        """Drag the given page up."""
        self._drag_page(direction='up')

    def drag_page_down(self):
        """Drag the given page down."""
        self._drag_page(direction='down')

    def _drag_page(self, direction):
        """Function to drag the page up/down."""
        self._wait_to_stop_moving()

        x, y, w, h = self.globalRect
        start_x = stop_x = x + (w / 2)
        start_y = y + (h / 2)

        if direction == "down":
            stop_y = start_y + h / 3
            self.pointing_device.drag(start_x, start_y, stop_x, stop_y)
        else:
            stop_y = start_y - h / 3
            self.pointing_device.drag(start_x, start_y, stop_x, stop_y)

        self._wait_to_stop_moving()

    def _wait_to_stop_moving(self):
        self.select_single(
            'QQuickFlickable',
            objectName='animationContainer').moving.wait_for(False)


class AlarmPage(Page):
    """Autopilot helper for the Alarm page."""

    def get_num_of_alarms(self):
        """Return the number of saved alarms."""
        return int(self.wait_select_single(
            toolkit_emulators.QQuickListView,
            objectName='listSavedAlarm').count)

    @autopilot_logging.log_action(logger.info)
    def add_alarm(self, name, alarm_type, day):
        add_alarm_page = self._click_add_alarm_button()
        add_alarm_page.fill_alarm_info(name, alarm_type, day)
        self._click_save()
        self._wait_to_stop_moving()

    def _click_add_alarm_button(self):
        """Click the add alarm toolbar button."""
        toolbar = self.main_view.open_toolbar()
        toolbar.click_button('addAlarmButton')
        return self.main_view.wait_select_single(AddAlarmPage)

    def _click_save(self):
        """Click the save alarm button."""
        toolbar = self.main_view.open_toolbar()
        toolbar.click_button('saveAlarmButton')

    @autopilot_logging.log_action(logger.info)
    def delete_alarm(self, index):
        """Delete an alarm at the specified index."""
        alarm = self.wait_select_single(
            toolkit_emulators.Base, objectName='alarm{}'.format(index))
        if not self.main_view.wideAspect:
            self.drag_page_up()
        alarm.swipe_to_delete()
        alarm.confirm_removal()

    @autopilot_logging.log_action(logger.info)
    def toggle_alarm(self, index):
        """Toggle an alarm on/off."""
        alarm = self.wait_select_single(
            toolkit_emulators.CheckBox,
            objectName='listAlarmStatus{}'.format(index))
        if not self.main_view.wideAspect:
            self.drag_page_up()
        self.pointing_device.click_object(alarm)


class AddAlarmPage(Page):
    """Autopilot helper for the Add Alarm page."""

    def fill_alarm_info(self, name, alarm_type, day):
        self._set_alarm_time()
        if not self.main_view.wideAspect:
            self.drag_page_up()
        self._set_alarm_name(name)
        self._click_alarm_hour_label()
        self._set_alarm_type(alarm_type)
        if not self.main_view.wideAspect:
            self.drag_page_up()
        self._set_alarm_trigger_day(day, alarm_type)

    def _set_alarm_name(self, name):
        name_alarm = self.wait_select_single(
            LabelDots, objectName='addAlarmName')
        name_alarm.write(name)

    def _set_alarm_time(self):
        self._click_alarm_hour_label()
        hour = self._get_alarm_hour_hand()

        x, y, w, h = hour.globalRect
        tx = x + (w // 2)
        ty = y + (h // 2.5)
        self.pointing_device.drag(
            tx, ty - (h // 4), tx - (w // 2), ty + (h // 2))

        self._click_alarm_minute_label()
        minute = self._get_alarm_minute_hand()

        x, y, w, h = minute.globalRect
        tx = x + (w // 2)
        ty = y + (h // 2.5)
        self.pointing_device.drag(
            tx, ty - (h // 4), tx + (w // 2), ty + (h // 2))

    def _click_alarm_hour_label(self):
        hour_label = self.wait_select_single('Label', objectName='hourLabel')
        self.pointing_device.click_object(hour_label)

    def _click_alarm_minute_label(self):
        minute_label = self.wait_select_single(
            'Label', objectName='minuteLabel')
        self.pointing_device.click_object(minute_label)

    def _get_alarm_hour_hand(self):
        """Return the hour hand of clock in add alarm page"""
        return self.wait_select_single('DialerHand', objectName='hourHand')

    def _get_alarm_minute_hand(self):
        """Returns the minute hand of clock in add alarm page"""
        return self.wait_select_single('DialerHand', objectName='minuteHand')

    def _set_alarm_type(self, alarm_type):
        alarm_type_checkbox = self.wait_select_single(
            toolkit_emulators.CheckBox, objectName='RepeatSelector')
        if alarm_type == 'Single':
            alarm_type_checkbox.uncheck()
        else:
            alarm_type_checkbox.check()

    def _set_alarm_trigger_day(self, day, alarm_type):
        """Set the day the alarm triggers on."""
        if alarm_type == 'Single':
            self._set_single_alarm_day(day)
        else:
            self._set_recurring_alarm_day()

    def _set_single_alarm_day(self, day):
        """Set the alarm day of a single type alarm."""
        # TODO: we need a proper UITK emulator for optionSelector
        # http://pad.lv/1277059  --nik90 2014-02-06
        day_option_selector = self.select_single(
            'OptionSelector', objectName='OccursSelector')
        days_object = day_option_selector.select_single(
            'StyledItem', objectName='listContainer')
        dayslist = days_object.select_many('Label', visible=True)

        # Click on days option selector to expand it
        self.pointing_device.click_object(dayslist[0])

        for index, day_element in enumerate(dayslist):
            if(day_element.text == day):
                day_index = index
                break
        else:
            raise ClockEmulatorException('Unknown day: {}'.format(day))

        if not self.main_view.wideAspect:
            self.drag_page_up()
        self.pointing_device.click_object(dayslist[day_index])

    def _set_recurring_alarm_day(self):
        """Set the alarm days of a recurring alarm."""
        dialog = self._open_recurring_alarm_dialog()
        dialog.set_alarm_days()

    def _open_recurring_alarm_dialog(self):
        """Open the recurring alarm dialogue."""
        recurring_option = self.wait_select_single(
            toolkit_emulators.MultiValue, objectName='OccurrenceList')
        self.pointing_device.click_object(recurring_option)
        return self.main_view.wait_select_single(
            AlarmDays, objectName='alarmDaysDialog')


class AlarmDays(toolkit_emulators.UbuntuUIToolkitEmulatorBase):
    """Autopilot helper for the Alarm Days dialog."""

    def set_alarm_days(self):
        self._fill_alarm_days()
        self._click_confirm_recurring_alarm_days()

    def _fill_alarm_days(self):
        # Get all the day objects in the column and enable only those
        # days whose index is an even number and disable the others
        # TODO receive as a parameter the list of days to enable.
        # --elopio - 2014-02-24
        for index in range(7):
            day_element = self.wait_select_single(
                toolkit_emulators.CheckBox,
                objectName='repeatDaysSwitch{}'.format(index))
            if index % 2 == 0:
                day_element.check()
            else:
                day_element.uncheck()

    def _click_confirm_recurring_alarm_days(self):
        """Click the Done button in recurring alarms dialog."""
        done_button = self.wait_select_single(
            'Button', objectName='confirmRepeatDaysButton')
        self.pointing_device.click_object(done_button)


class TimerPage(Page):
    """Autopilot helper for the Timer page."""

    def get_num_of_presets(self):
        """Return the number of presets in the timer page."""
        return int(self.wait_select_single('QQuickListView').count)

    @autopilot_logging.log_action(logger.info)
    def add_timer(self, name, set_time=False):
        self._click_add_timer_button()
        self._wait_to_stop_moving()
        if not self.main_view.wideAspect:
            self.drag_page_up()
        self._set_timer_name(name)
        if not self.main_view.wideAspect:
            self.drag_page_down()
        if set_time:
            self._set_time()
        self._click_save()

    def _click_add_timer_button(self):
        toolbar = self.main_view.open_toolbar()
        toolbar.click_button('addPresetButton')

    def _set_timer_name(self, name):
        name_label = self.wait_select_single(
            LabelDots, objectName='namePreset')
        name_label.write(name)
        if name_label.text != name:
            raise ClockEmulatorException('Failed to write the timer name.')

    def _set_time(self):
        # Set timer minute
        logger.debug('Grabbing minute hand')

        timeout = 0
        timer_face = self.select_single('TimerFace')
        #need to select clock in order to set time
        while not timer_face.focus and timeout < 10:
            logger.debug('Timerlabel loop')
            self._click_timer_label()
            sleep(1)
            timeout += 1

        # TODO we need a dialer hand emulator, with proper self-tests.
        # --elopio -2014-01-31
        minute = self._get_timer_minute_hand()
        #make sure timer is set
        x, y, w, h = minute.globalRect
        tx = x + (w / 2)
        ty = y + (h / 2.5)
        timeout = 0
        reset_button = self._get_timer_reset_button()
        logger.debug('reset_button %s' % reset_button.visible)

        #Try again if we failed at first
        while not reset_button.visible and timeout < 10:
            logger.debug('Dragging minute hand')
            self.pointing_device.drag(
                tx, ty - (h / 4), tx + (w / 2), ty + (h / 2))
            sleep(1)
            timeout += 1

    def _get_timer_minute_hand(self):
        """Return the hour hand of clock in timer tab."""
        return self.wait_select_single('DialerHand', objectName='minuteHand')

    def _click_timer_label(self):
        timer_label = self._get_label_timer()
        self.pointing_device.click_object(timer_label)

    def _get_label_timer(self):
        """Return the label with the timer countdown."""
        return self.select_single('Label', objectName='label')

    def _get_timer_reset_button(self):
        """Return the select for the reset button."""
        return self.select_single('ImageButton', objectName='resetButtonTimer')

    def _click_save(self):
        toolbar = self.main_view.open_toolbar()
        toolbar.click_button('savePresetButton')

    def get_presets(self):
        """Return a list with the information of the presets.

        Each item of the returned list is a tuple of (name, time).

        """
        presets = []
        for index in range(self.get_num_of_presets()):
            text = self.select_single(
                'Label', objectName='presetTextLabel{}'.format(index)).text
            time = self.select_single(
                'Label', objectName='presetTimerLabel{}'.format(index)).text
            presets.append((text, time))
        return presets

    @autopilot_logging.log_action(logger.info)
    def run_preset_timer(self, index):
        self._select_preset(index)
        self._click_timer_label()

    def _select_preset(self, index):
        presets_list = self.select_single(
            toolkit_emulators.QQuickListView, objectName='listTimerPreset')
        presets_list.click_element('preset{}'.format(index))

    def get_current_time(self):
        """Return the time of the currently selected timer."""
        self._get_label_timer().text

    @autopilot_logging.log_action(logger.info)
    def delete_preset_timer(self, index):
        preset = self.select_single(
            toolkit_emulators.Standard, objectName='preset{}'.format(index))
        preset.swipe_to_delete()
        preset.confirm_removal()
