# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013, 2014 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authored by: Nicholas Skaggs <nicholas.skaggs@canonical.com>
#              Nekhelesh Ramananthan <krnekhelesh@gmail.com>

"""Tests for the Clock App - Alarm"""

from __future__ import absolute_import

import datetime
import unittest

from autopilot.matchers import Eventually
from testtools.matchers import Equals, NotEquals
from ubuntu_clock_app.tests import ClockAppTestCase


class TestAlarm(ClockAppTestCase):
    """Tests the alarm page features"""

    def setUp(self):
        """ This is needed to wait for the application to start.

        In the testfarm, the application may take some time to show up.

        """
        super(TestAlarm, self).setUp()
        self.assertThat(
            self.main_view.visible, Eventually(Equals(True)))

        self.page = self.main_view.open_alarm()

    def test_add_single_type_alarm_must_add_to_alarm_list(self):
        """Test to check if a single type alarm is saved properly

        This test saves a single type alarm and verifies if it is added to the
        alarm list in the alarm page.

        """
        # The alarm day is always set to tomorrow to ensure that it is
        # not rejected by the clock app since the clock app does not
        # allow alarms to be created before the current time of the same
        # day. This limitation will be removed later.
        # -- nik90 2014-02-06
        now = datetime.datetime.now() + datetime.timedelta(days=1)
        tomorrow_day = now.strftime("%A")
        old_alarm_count = self.page.get_num_of_alarms()
        self.page.add_alarm('Single Test', 'Single', tomorrow_day)

        # Check if the alarm count has increased
        self.assertThat(self.page.get_num_of_alarms,
                        Eventually(NotEquals(old_alarm_count)))

        #TODO: Check if the newly saved alarm has the properties set by
        # add_alarm method

    def test_add_recurring_type_alarm_must_add_to_alarm_list(self):
        """ Test to check if a recurring alarm is saved properly

        Recurring alarms are triggered on more than one day. This test saves a
        recurring alarm and check if it has been saved and is added to the
        alarm list in the alarm page.

        """
        old_alarm_count = self.page.get_num_of_alarms()
        self.page.add_alarm('Recurring Test', 'Recurring', 'Null')

        # Check if the alarm count has increased
        self.assertThat(self.page.get_num_of_alarms,
                        Eventually(NotEquals(old_alarm_count)))

    def test_delete_alarm_must_delete_from_alarm_list(self):
        """Test to delete an alarm and check if it has been properly removed"""
        # FIXME: Instead of always deleting the first alarm, run the alarm
        # which was created. To do this in a nice way, we should use the
        # QQuickListView but it has a bug that makes it fail in this case:
        # http://pad.lv/1275060 --elopio - 2014-01-31

        # (set-up) Create a recurring alarm
        self.page.add_alarm('Recurring Test', 'Recurring', 'Null')

        old_alarm_count = self.page.get_num_of_alarms()
        self.page.delete_alarm(index=0)

        # Check that the alarm is deleted
        self.assertThat(self.page.get_num_of_alarms,
                        Eventually(Equals(old_alarm_count - 1)))

    @unittest.skip("Pending development, bug 1272337")
    def test_toggle_alarm_status_must_enable_or_disable_alarm(self):
        """Test to toggle an alarm and check it has been done properly"""
        # FIXME: This test is skipped due to an upstream bug in EDS which
        # causes the alarm status to not be properly toggled. This could
        # result in flaky test.
        # http://pad.lv/1272337 -- nik90 - 2014-02-07

        # (set-up) Create a recurring alarm
        self.page.add_alarm('Recurring Test', 'Recurring', 'Null')

        self.page.toggle_alarm(index=0)
